/*
 * ConnectedComponent.java
 *
 * Copyright (c) 2006-2007 Computer-Aided Integrated Systems Laboratory, 
 * St. Petersburg Institute for Informatics and Automation RAS http://cais.lisa.iias.spb.su
 *
 * This software is open-source under the BSD license; see either "license.txt" 
 * or http://jung.sourceforge.net/license.txt for a description.
 */

package connectedsubgraph;

import edu.uci.ics.jung.graph.Graph;
import edu.uci.ics.jung.graph.Vertex;
import edu.uci.ics.jung.graph.Edge;

import java.util.Map;
import java.util.Set;
import java.util.HashSet;
import java.util.List;
import java.util.ArrayList;

/** Search of graph connected components
 */
public class ConnectedComponent {
    
    private Set<Vertex> v_checked;
    
    public ConnectedComponent() {
        v_checked = new HashSet<Vertex>();
        clearCheckedVertices();
    }
    
    /** Clear marker for checked vertices.
     */
    public void clearCheckedVertices() {
        v_checked.clear();
    }
    
    
    /** Maps list of sets of vertices via the map of vertices 'm'.
     * There are (1) the sets of vertices (l_source), (2) map from (l_source)
     * to (l_dest).
     *
     * @return sets of vertices (l_dest) corresponding to the source sets of vertices
     */
    public static List<Set<Vertex>> mapSetToSet(List<Set<Vertex>> l_source,Map<Vertex, Vertex> m) {
        
        Set<Vertex>         s_dest;
        List<Set<Vertex>>   l_dest = new ArrayList<Set<Vertex>>();
        
        for(Set<Vertex> s_source:l_source) {
            if(null != s_source && 0 < s_source.size()) {
                
                s_dest = new HashSet<Vertex> ();
                for(Vertex v:s_source) {
                    s_dest.add(m.get(v));
                }
                l_dest.add(s_dest);
                
            }
        }
        return l_dest;
    }
    
    /** Gets vertices of the connected component that contains 
     * the vertex 'v_source'.
     *
     * Remark: use <tt>{@link clearCheckedVertices}</tt> if new graph 'g'
     * is tested.
     */
    @SuppressWarnings("unchecked")
    public Set<Vertex> getConnectedComponent (Vertex v_source, Graph g) {
        
        if(null == v_source || null == g)
            return null;
        
        Set<Vertex> res = new HashSet<Vertex>();
        Set<Vertex> vert = g.getVertices();
        if(!vert.contains(v_source))
            return null;
        
        List<Vertex> stack = new ArrayList<Vertex>();
        
        // Depth-first search (DFS)
        
        stack.add(v_source);
        while(0 < stack.size()) {
            Vertex v = stack.remove(0);
            
            res.add(v);
            v_checked.add(v);
            
            Set<Vertex> v_neighbours = v.getNeighbors();
            for(Vertex n:v_neighbours) {
                if(!stack.contains(n) && !v_checked.contains(n)) {
                    stack.add(n);
                }
            }
        }
        
        return res;
    }
    
    /** Gets connected components of the graph 'g_cur' */
    @SuppressWarnings("unchecked")
    public List<Set<Vertex>> getConnectedComponents (Graph g_cur) {
        
        List<Set<Vertex>> cc = new ArrayList<Set<Vertex>>();
        
        clearCheckedVertices();
        for(Object o:g_cur.getVertices()) {
            Vertex v = (Vertex)o;
            if(!v_checked.contains(v)) {
                cc.add( getConnectedComponent(v, g_cur) );
            }
        }
        //return (Set<Vertex>[])cc.toArray((HashSet<Vertex>[])new HashSet[0]);
        return cc;
    }

    
}
