/*
 * GraphCreator.java
 *
 * Copyright (c) 2006 Computer-Aided Integrated Systems Laboratory, 
 * St. Petersburg Institute for Informatics and Automation RAS http://cais.lisa.iias.spb.su
 *
 * This software is open-source under the BSD license; see either "license.txt" 
 * or http://jung.sourceforge.net/license.txt for a description.
 */

package matching;

import edu.uci.ics.jung.utils.UserData;
import edu.uci.ics.jung.graph.Graph;
import edu.uci.ics.jung.graph.impl.SparseGraph;
import edu.uci.ics.jung.graph.impl.SparseVertex;
import edu.uci.ics.jung.graph.impl.UndirectedSparseEdge;
import edu.uci.ics.jung.graph.Edge;
import edu.uci.ics.jung.graph.Element;
import edu.uci.ics.jung.graph.Vertex;

/** Creates graphs from array of vertices' names and adjacent matrix. 
 * Has several test graphs.
 */
public class GraphCreator {
    
    /** Creates a new instance of GraphCreator */
    public GraphCreator() {
    }

    
    
    public static Graph createGraph(String[] labels, int[][] edges) {
        if(null == labels) {
            return null;
        }
        Graph graph = new SparseGraph();
        
        //v_graph = new ArrayList<Vertex>();
        
        Vertex[] v = new Vertex[labels.length];
        for (int i = 0; i < labels.length; i++) {
            v[i] = graph.addVertex(new SparseVertex());
            v[i].addUserDatum(GraphComparator.NAME_KEY, labels[i], UserData.SHARED);
        }
        
        if(null != edges) {
            for(int i=0; i<edges.length; i++) {
                graph.addEdge(new UndirectedSparseEdge(v[edges[i][0]], v[edges[i][1]]));
            }
        }
        return graph;
    }
    
    /** Adds new vertices (named by labels) and edges to the graph */
    public static Graph addGraph(Graph g,String[] labels, int[][] edges) {
        Graph graph = new SparseGraph();
        
        for(Object o:g.getVertices()) {
            Vertex v = graph.addVertex(new SparseVertex());
            String name = (String)((Vertex)o).getUserDatum(GraphComparator.NAME_KEY);
            v.addUserDatum(GraphComparator.NAME_KEY, name, UserData.SHARED);
            v.addUserDatum(GraphComparator.SUBGRAPH_N, "1", UserData.SHARED);
        }
        for(Object o:g.getEdges()) {
            Edge e = ((Edge)o);
            String name1 = (String)((Vertex)e.getEndpoints().getFirst()).getUserDatum(GraphComparator.NAME_KEY);
            String name2 = (String)((Vertex)e.getEndpoints().getSecond()).getUserDatum(GraphComparator.NAME_KEY);
            Vertex v1 = VertexPairSimilar.getVertexByName(graph,name1);
            Vertex v2 = VertexPairSimilar.getVertexByName(graph,name2);
            graph.addEdge(new UndirectedSparseEdge(v1, v2));
        }
        
        Vertex[] v = new Vertex[labels.length];
        for(int i = 0; i < labels.length; i++) {
            v[i] = graph.addVertex(new SparseVertex());
            v[i].addUserDatum(GraphComparator.NAME_KEY, labels[i], UserData.SHARED);
        }
        
        for(int i=0; i<edges.length; i++) {
            graph.addEdge(new UndirectedSparseEdge(v[edges[i][0]], v[edges[i][1]]));
        }
        return graph;
    }
    
    
    /*  Test data 1                  Test data 2

              | 0 Thing                    | 0 Thing
              |                            |
              | 1 Supplier                 | 1 Provider
             /`-.                         /`-.
           .'    `-.                    .'    `-.
          /         `-2 Hardware       /         /-2 Computer
     3 Furniture    .'               3 Furniture/
                   / 
                 .4 Mobile-Phone
     
     *      Distance (Thing, Thing) = 0;
     *      Distance (Furniture, Furniture) = 0;
     *      Distance (Supplier, Computer) = 5;
     */
    public static final String[] labels1 = {
            // 0-4 vertices
            "Thing",        // 0
            "Supplier",     // 1
            "Hardware",     // 2
            "Furniture",    // 3
            "Mobile-Phone", // 4
        };
    public static int[][] edges1 = { {0,1}, {1,2}, {1,3}, {2,4}};
    
    public static final String[] labels2 = {
            // 5-9 vertices
            "Thing",        // 5
            "Provider",     // 6
            "Computer",     // 7
            "Furniture"     // 8
        };
    
    public static int[][] edges2 = { {0,1}, {1,2}, {1,3}, {2,3}};
    
    
    
    /*  Unconnected graphs
         Test data 15                  Test data 25

              | 0 Thing                    | 0 Thing
              |                            |
              | 1 Supplier                 | 1 Provider
             /`-.                         /`\ 
           .'    `-.                    .'    \_ 
          /         `-2 Hardware       /        ^--2 Computer
     3 Furniture                     3 Furniture-/ 
                                                      
                 .4 Mobile-Phone                        4 Calculator
                 |                                      
                 .5 Communicator                        5 LED
     
     */
    public static final String[] labels15 = {
            // 0-4 vertices
            "Thing",        // 0
            "Supplier",     // 1
            "Hardware",     // 2
            "Furniture",    // 3
            "Mobile-Phone", // 4
            "Communicator", // 5
        };
    public static int[][] edges15 = { {0,1}, {1,2}, {1,3}, {4,5}};
    
    public static final String[] labels25 = {
            // 5-9 vertices
            "Thing",        // 0  5
            "Provider",     // 1  6
            "Computer",     // 2  7
            "Furniture",    // 3  8
            "Calculator",   // 4  9
            "LED",          // 5  10
        };
    
    public static int[][] edges25 = { {0,1}, {1,2}, {2,3}, {3,1}};  // {4,5}
    
    
    // ---
    // Test data 3
    // 0 -- 1 -- 2
    // ---
    
    public final String[] labels3 = {
            // 0-3 vertices
            "node1",        // 0
            "node2",        // 1
            "node3"         // 2
        };
    public int[][] edges3 = { {0,1}, {1,2}};
    
    /* Test data 4
                  .'|node2
                .'  |`.
               /    |  `.
             .'     |    `.
           .node1   |      :node3
           `-.      |    .'
              `-._  | .-'
                  `-|node4
     */
    public final String[] labels4 = {
            // 0-4 vertices
            "node1",        // 0
            "node2",        // 1
            "node3",        // 2
            "node4"         // 3
        };
    
    public int[][] edges4 = { {0,1}, {1,2}, {2,3}, {3,0}, {1,3}};
}
