/*
 * Copyright (c) 2006-2007 Computer-Aided Integrated Systems Laboratory, 
 * St. Petersburg Institute for Informatics and Automation RAS http://cais.lisa.iias.spb.su
 *
 * This software is open-source under the BSD license; see either "license.txt" 
 * or http://jung.sourceforge.net/license.txt for a description.
 */

package matching;

import java.net.URL;
import javax.swing.JComboBox;
import javax.swing.BoxLayout;
import javax.swing.Box;
import javax.swing.JButton;
import java.awt.event.KeyEvent;
import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.JFileChooser;
import java.io.InputStream;
import java.io.File;
import java.io.IOException;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JApplet;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;

import edu.uci.ics.jung.io.PajekNetReader;
import edu.uci.ics.jung.io.PajekNetWriter;
import edu.uci.ics.jung.graph.Graph;
import edu.uci.ics.jung.graph.decorators.DefaultToolTipFunction;
import edu.uci.ics.jung.graph.decorators.PickableEdgePaintFunction;
import edu.uci.ics.jung.utils.TestGraphs;
import edu.uci.ics.jung.visualization.DefaultVisualizationModel;
import edu.uci.ics.jung.visualization.FRLayout;
import edu.uci.ics.jung.visualization.GraphZoomScrollPane;
import edu.uci.ics.jung.visualization.ISOMLayout;
import edu.uci.ics.jung.visualization.Layout;
import edu.uci.ics.jung.visualization.MultiPickedState;
import edu.uci.ics.jung.visualization.PickSupport;
import edu.uci.ics.jung.visualization.PickedState;
import edu.uci.ics.jung.visualization.PluggableRenderer;
import edu.uci.ics.jung.visualization.ShapePickSupport;
import edu.uci.ics.jung.visualization.VisualizationModel;
import edu.uci.ics.jung.visualization.VisualizationViewer;
import edu.uci.ics.jung.visualization.control.CrossoverScalingControl;
import edu.uci.ics.jung.visualization.control.DefaultModalGraphMouse;
import edu.uci.ics.jung.visualization.control.ScalingControl;
import edu.uci.ics.jung.visualization.transform.MutableTransformer;

/** Draws two graphs with 2 layouts in 2 views.
 * Loads/saves in Pajek format. Compares two graphs.
 */
public class TwoModelCompare extends JApplet {
    
    Graph graph1;
    Graph graph2;
    Layout layout1;
    Layout layout2;

    /**
     * the visual components and renderers for the graphs
     */
    VisualizationViewer vv1;
    VisualizationViewer vv2;
    
    /** Directory contains first graph in Pajek format. */
    String pajek_dir1;
    String pajek_dir2;
    JFileChooser fc1;
    JFileChooser fc2;
    JTextField pajek_dir_field1;
    JTextField pajek_dir_field2;
    JPanel panel;
    JTextField sim_value_field1;
    
    PajekNetWriter pnw;
    PajekNetReader pnr;    
        
    /**
     * the normal transformer
     */
    MutableTransformer layoutTransformer;
    
    Dimension preferredSize = new Dimension(300,300);
    
    
    GraphComparator gc;
    LevenshteinDistance lev;
    
    
    /**
     * create an instance of a simple graph in two views with controls to
     * demo the zoom features.
     * 
     */
    public TwoModelCompare() {
        
        String[] pajek_filenames = { "4_triangle_computer.net", "5_mobile_phone.net", "graph_first.net", 
                                    "Graph1.net", "Graph2.net", "simple.net", "smyth.net" };
        
        // calculates initial similarity
        lev = new LevenshteinDistance();
        lev.use_dist_matrix = true;
        lev.setContstraintType(LevenshteinDistance.TYPE_SOFT);
        gc = new GraphComparator();
        
        pnw = new PajekNetWriter();
        pnr = new PajekNetReader();
        //pnw.save(graph1, testFilename, gl, null, null);
        //pnw.save(graph2, testFilename2, new PajekLabels(), null, null);
        
        
        // create a simple graph for the demo
        // both models will share one graph
        graph1 = TestGraphs.getOneComponentGraph();
        graph2 = TestGraphs.getOneComponentGraph(); // getDemoGraph();
        
        // both views will share the same renderer
        PluggableRenderer pr1 = new PluggableRenderer();
        PluggableRenderer pr2 = new PluggableRenderer();
        
        UnicodeVertexStringer u_vertexstringer = new UnicodeVertexStringer();
        pr1.setVertexStringer(u_vertexstringer);
        pr2.setVertexStringer(u_vertexstringer);
        
        // create two layouts for the one graph, one layout for each model
        layout1 = new FRLayout(graph1);
        layout2 = new ISOMLayout(graph2);

        // create the two models, each with a different layout
        VisualizationModel vm1 =
            new DefaultVisualizationModel(layout1, preferredSize);
        VisualizationModel vm2 = 
            new DefaultVisualizationModel(layout2, preferredSize);


        // create the two views, one for each model
        vv1 = new VisualizationViewer(vm1, pr1, preferredSize);
        vv2 = new VisualizationViewer(vm2, pr2, preferredSize);
        
        // share the model transformer between the two models
        layoutTransformer = vv1.getLayoutTransformer();
        vv2.setLayoutTransformer(layoutTransformer);
        
        // share the view transformer between the two models
        vv2.setViewTransformer(vv1.getViewTransformer());

        vv1.setBackground(Color.white);
        vv2.setBackground(Color.white);
        
        // create and set a separate pickSupport for each view
        PickSupport pickSupport1 = new ShapePickSupport();
        vv1.setPickSupport(pickSupport1);
        PickSupport pickSupport2 = new ShapePickSupport();
        vv2.setPickSupport(pickSupport2);

        // share one PickedState between the two views
        PickedState ps = new MultiPickedState();
        vv1.setPickedState(ps);
        vv2.setPickedState(ps);
        
        // set an edge paint function that will show picking for edges
        pr1.setEdgePaintFunction(new PickableEdgePaintFunction(ps, Color.black, Color.red));
        pr2.setEdgePaintFunction(new PickableEdgePaintFunction(ps, Color.black, Color.red));

        // add default listeners for ToolTips
        vv1.setToolTipFunction(new DefaultToolTipFunction());
        vv2.setToolTipFunction(new DefaultToolTipFunction());
        
        Container content = getContentPane();
        panel = new JPanel(new GridLayout(1,0));
        panel.add(new GraphZoomScrollPane(vv1));
        panel.add(new GraphZoomScrollPane(vv2));

        content.add(panel);
        
        // create a GraphMouse for each view
        final DefaultModalGraphMouse gm1 = new DefaultModalGraphMouse();

        DefaultModalGraphMouse gm2 = new DefaultModalGraphMouse();

        vv1.setGraphMouse(gm1);
        vv2.setGraphMouse(gm2);
        
        
        // bf - browse file panel
        // "Graph Pajek file (.net)" [File chooser]
        JPanel jp_bf = new JPanel();
        jp_bf.setLayout(new BoxLayout(jp_bf, BoxLayout.X_AXIS));
        
        JLabel pajek_label1 = new JLabel("Graph 1 Pajek file ( .net)");//, Label.RIGHT)
        
        pajek_dir_field1 = new JTextField();
        //pajek_dir = wb.parameters.getLogDir();
        pajek_dir1 = System.getProperty("user.home");
        pajek_dir_field1.setText(pajek_dir1);
        
        
        // cb1 - pajek file list combo box 1
        JComboBox cb1 = new JComboBox(pajek_filenames);
        cb1.setSelectedIndex(1);
        cb1.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                JComboBox cb = (JComboBox)e.getSource();
                String fname = (String)cb.getSelectedItem();
                
                pajek_dir_field1.setText(fname);
                try{
                    graph1 = pnr.loadFromJAR(fname);
                    GraphComparator.reloadPajekVertexLabelsToGraph(graph1);
                    System.out.println("vertices="+graph1.numVertices()+"; edges="+graph1.numEdges());
                    layout1 = new FRLayout(graph1);
                    vv1.stop();
                    vv1.setGraphLayout(layout1);
                    vv1.restart();
                }
                catch (IOException ioe) {
                    System.out.println("TwoModelCompare: unexpected IOException");
                }
                double r = gc.compare(graph1, graph2, lev);
                sim_value_field1.setText("" + r);
            }
        });
        
        
        // browse log file button
        JButton browse_btn1 = new JButton("Browse...");           //browse_btn1.setAlignmentX(0.5f);
        browse_btn1.setToolTipText("Browse directory for .net file (graph in Pajek format)");
        browse_btn1.setMnemonic(KeyEvent.VK_B);
        browse_btn1.setBackground(Color.decode("#D8C0C0"));
        
        fc1 = new JFileChooser();
        
        browse_btn1.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (JFileChooser.APPROVE_OPTION == fc1.showOpenDialog(panel)) {
                    File file = fc1.getSelectedFile();
                    pajek_dir1 = file.getAbsolutePath();
                    pajek_dir_field1.setText(pajek_dir1);
                    
                    System.out.println("getAbsolutePath()="+file.getAbsolutePath());
                    try{
                        graph1 = pnr.load(file.getAbsolutePath());
                        GraphComparator.reloadPajekVertexLabelsToGraph(graph1);
                        System.out.println("vertices="+graph1.numVertices()+"; edges="+graph1.numEdges());
                        layout1 = new FRLayout(graph1);
                        vv1.stop();
                        vv1.setGraphLayout(layout1);
                        vv1.restart();
                    }
                    catch (IOException ioe) {
                        System.out.println("TwoModelCompare: unexpected IOException");
                    }
                } else { // print old path
                    pajek_dir_field1.setText(pajek_dir1);
                }
            }
        });
        
        jp_bf.add(pajek_label1);
        jp_bf.add(pajek_dir_field1);
        jp_bf.add(Box.createRigidArea(new Dimension(5,0)));
        jp_bf.add(cb1);
        jp_bf.add(Box.createRigidArea(new Dimension(5,0)));
        jp_bf.add(browse_btn1);
        jp_bf.add(Box.createRigidArea(new Dimension(5,0)));
        
        
        JPanel jp_bf2 = new JPanel();
        jp_bf2.setLayout(new BoxLayout(jp_bf2, BoxLayout.X_AXIS));
        
        JLabel pajek_label2 = new JLabel("Graph 2 Pajek file ( .net)");
        
        pajek_dir_field2 = new JTextField();
        pajek_dir2 = System.getProperty("user.home");
        pajek_dir_field2.setText(pajek_dir2);
        
        
        // browse file button
        // cb2 - pajek file list combo box 2
        JComboBox cb2 = new JComboBox(pajek_filenames);
        cb2.setSelectedIndex(0);
        cb2.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                JComboBox cb = (JComboBox)e.getSource();
                String fname = (String)cb.getSelectedItem();
                
                pajek_dir_field2.setText(fname);
                try{
                    graph2 = pnr.loadFromJAR(fname);
                    GraphComparator.reloadPajekVertexLabelsToGraph(graph2);
                    System.out.println("vertices="+graph2.numVertices()+"; edges="+graph2.numEdges());
                    layout2 = new FRLayout(graph2);
                    vv2.stop();
                    vv2.setGraphLayout(layout2);
                    vv2.restart();
                }
                catch (IOException ioe) {
                    System.out.println("TwoModelCompare: unexpected IOException");
                }
                double r = gc.compare(graph1, graph2, lev);
                sim_value_field1.setText("" + r);
            }
        });
        
        
        JButton browse_btn2 = new JButton("Browse...");           //browse_btn1.setAlignmentX(0.5f);
        browse_btn2.setToolTipText("Browse directory for .net file (graph in Pajek format)");
        browse_btn2.setMnemonic(KeyEvent.VK_R);
        browse_btn2.setBackground(Color.decode("#D8C0C0"));
        
        fc2 = new JFileChooser();
        
        browse_btn2.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (JFileChooser.APPROVE_OPTION == fc2.showOpenDialog(panel)) {
                    File file = fc2.getSelectedFile();
                    pajek_dir2 = file.getAbsolutePath();
                    pajek_dir_field2.setText(pajek_dir2);
                    
                    System.out.println("getAbsolutePath()="+file.getAbsolutePath());
                    try{
                        graph2 = pnr.load(file.getAbsolutePath());
                        GraphComparator.reloadPajekVertexLabelsToGraph(graph2);
                        System.out.println("vertices="+graph2.numVertices()+"; edges="+graph2.numEdges());
                        layout2 = new FRLayout(graph2);
                        vv2.stop();
                        vv2.setGraphLayout(layout2);
                        vv2.restart();
                    }
                    catch (IOException ioe) {
                        System.out.println("TwoModelCompare: unexpected IOException");
                    }
                } else { // print old path
                    pajek_dir_field2.setText(pajek_dir2);
                }
            }
        });
        
        jp_bf2.add(pajek_label2);
        jp_bf2.add(pajek_dir_field2);
        jp_bf2.add(Box.createRigidArea(new Dimension(5,0)));
        jp_bf2.add(cb2);
        jp_bf2.add(Box.createRigidArea(new Dimension(5,0)));
        jp_bf2.add(browse_btn2);
        jp_bf2.add(Box.createRigidArea(new Dimension(5,0)));
        
        
        // "Graphs similarity is:" [JTextField] [JButton "Calculate"]
        JPanel jp_sim_value = new JPanel();
        jp_sim_value.setLayout(new BoxLayout(jp_sim_value, BoxLayout.X_AXIS));
        JLabel sim_label1 = new JLabel("Graphs similarity is:");
        sim_value_field1 = new JTextField();
        
        // browse log file button
        JButton calc_btn1 = new JButton("Calculate");
        calc_btn1.setToolTipText("Calculates similarity between graph 1 and 2");
        calc_btn1.setMnemonic(KeyEvent.VK_C);
        calc_btn1.setBackground(Color.decode("#D8C0C0"));
        
        calc_btn1.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                double r = gc.compare(graph1, graph2, lev);
                sim_value_field1.setText("" + r);
                }
        });
        
        jp_sim_value.add(sim_label1);
        jp_sim_value.add(Box.createRigidArea(new Dimension(5,0)));
        jp_sim_value.add(sim_value_field1);
        jp_sim_value.add(Box.createRigidArea(new Dimension(5,0)));
        jp_sim_value.add(calc_btn1);
        jp_sim_value.add(Box.createRigidArea(new Dimension(5,0)));
        
        
        
        JPanel jp_top = new JPanel();
        jp_top.setLayout(new BoxLayout(jp_top, BoxLayout.Y_AXIS));
        
        jp_top.add(jp_bf);
        jp_top.add(Box.createRigidArea(new Dimension(0,5)));
        jp_top.add(jp_bf2);
        jp_top.add(Box.createRigidArea(new Dimension(0,5)));
        jp_top.add(jp_sim_value);
        
        
        // create zoom buttons for scaling the transformer that is
        // shared between the two models.
        final ScalingControl scaler = new CrossoverScalingControl();

        JButton plus = new JButton("+");
        plus.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                scaler.scale(vv1, 1.1f, vv1.getCenter());
            }
        });
        JButton minus = new JButton("-");
        minus.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                scaler.scale(vv1, 0.9f, vv1.getCenter());
            }
        });
        
        JPanel zoomPanel = new JPanel(new GridLayout(1,2));
        zoomPanel.setBorder(BorderFactory.createTitledBorder("Zoom"));
        
        JPanel modePanel = new JPanel();
        modePanel.setBorder(BorderFactory.createTitledBorder("Mouse Mode"));
//        gm1.getModeComboBox().addItemListener(gm2.getModeListener());
        modePanel.add(gm1.getModeComboBox());

        JPanel controls = new JPanel();
        zoomPanel.add(plus);
        zoomPanel.add(minus);
        controls.add(zoomPanel);
        controls.add(modePanel);
        content.add(jp_top, BorderLayout.NORTH);
        content.add(controls, BorderLayout.SOUTH);
    }

    /**
     * a driver for this demo
     */
    public static void main(String[] args) {
        JFrame f = new JFrame();
        f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        f.getContentPane().add(new TwoModelCompare());
        f.pack();
        f.setVisible(true);
    }
}
