/*
 * Copyright (c) 2006-2007 Computer-Aided Integrated Systems Laboratory, 
 * St. Petersburg Institute for Informatics and Automation RAS http://cais.lisa.iias.spb.su
 *
 * This software is open-source under the BSD license; see either "license.txt" 
 * or http://jung.sourceforge.net/license.txt for a description.
 */

package matching;

import edu.uci.ics.jung.graph.Vertex;
import edu.uci.ics.jung.graph.Graph;
import edu.uci.ics.jung.algorithms.shortestpath.UnweightedShortestPath;

import java.util.Map;

/** Set of parameters to calculate similarity between two vertices */
public class VertexPairSimilar {
    Vertex v1, v2;
    /** Distance between names of vertices v1 and v2 */
    int string_sim;
    /** Number of neighbours vertices of v1 */
    int v1_neighbours;
    /** Number of neighbours vertices of v2 */
    int v2_neighbours;
    
    /** map<v_j,len_j>, where len_j is a length of shortest path from v1 to v_j */
    Map<Vertex,Integer> v1_path_len;
    /** map<v_j,len_j>, where len_j is a length of shortest path from v2 to v_j */
    Map<Vertex,Integer> v2_path_len;
    
    /** result similarity value between vertices v1 and v2 */
    private double d_sim;
    
    /** Gets similarity value between vertices v1 and v2 */
    public double getSimilarity () {
        return d_sim;
    }
    
    public VertexPairSimilar (Vertex ve1,Vertex ve2,
                //UnweightedShortestPath usp1, UnweightedShortestPath usp2,
                int title_sim,
                Map<Vertex,Vertex> similar_v1_v2,       Map<Vertex,Vertex> similar_v2_v1,
                int ve1_neighbours,                     int ve2_neighbours, 
                Map<Vertex,Integer> ve1_path_len,       Map<Vertex,Integer> ve2_path_len,
                Map<Vertex,Integer> v1_path_len_real,   Map<Vertex,Integer> v2_path_len_real) 
    {
        v1 = ve1;
         v2 = ve2;
          v1_neighbours = ve1_neighbours;
           v2_neighbours = ve2_neighbours;
            v1_path_len = ve1_path_len;
             v2_path_len = ve2_path_len;
        string_sim = title_sim;
        calcTwoVertexSimilarity(similar_v1_v2, similar_v2_v1, v1_path_len_real, v2_path_len_real);
    }
    
    public boolean isDifferLens(Vertex v1, Vertex v2, Map<Vertex,Integer> v1_path_len_real, Map<Vertex,Integer> v2_path_len_real) 
    {
        if (v1_path_len_real.containsKey(v1) && v2_path_len_real.containsKey(v2)) 
        {   
            int real_len1 = v1_path_len_real.get(v1);
            int real_len2 = v2_path_len_real.get(v2);

            if(real_len1 == real_len2) {
                return false;
            }
        }    
        return true;
    }
    /** Calculates result similarity value between vertices v1 and v2 
     * using class properties.
     * Takes into account the case:
     *   let's v1 is similar to v1', v2 is similar to v2', 
     *         v1 and v2 from graph 1, v1' and v2' from graph 1,
     *   if shortest_path_len(v1, v2) == shortest_path_len(v1', v2') then
     *      skip these vertices (even if the shortest path contains similar vertices).
     * @param similar_v1_v2 mapping of similar vertices (by name) Map <vertex_from_graph1, vertex_from_graph2>
     * @param similar_v2_v1 mapping vertices of graph2 to vertices of graph1
     * //@param usp1 unweighted shortest path structure for the graph 1
     * //@param usp2 unweighted shortest path structure for the graph 2
     */ 
    public double calcTwoVertexSimilarity(// UnweightedShortestPath usp1, UnweightedShortestPath usp2,
                        Map<Vertex,Vertex> similar_v1_v2,       Map<Vertex,Vertex> similar_v2_v1,
                        Map<Vertex,Integer> v1_path_len_real,   Map<Vertex,Integer> v2_path_len_real) {
        
        boolean real_len_differ;
        int sum; // sum of difference of shortest distances from v1 and v2 to similar vertices
        
        sum = 0;
        for(Vertex g1_v:v1_path_len.keySet()) { // g1_v - vertex in graph1
            Vertex g2_v = similar_v1_v2.get(g1_v);
            
            real_len_differ = isDifferLens(g1_v, g2_v, v1_path_len_real, v2_path_len_real);
            
            if(real_len_differ) {
                int len1 = v1_path_len.get(g1_v);   // length (v1, g1_v)
                int len2 = v2_path_len.containsKey(g2_v) ? v2_path_len.get(g2_v) : 0;
                sum += Math.abs(len1 - len2);
            }
        }
        
        for(Vertex g2_v:v2_path_len.keySet()) {
            Vertex g1_v = similar_v2_v1.get(g2_v);
            
            real_len_differ = isDifferLens(g1_v, g2_v, v1_path_len_real, v2_path_len_real);
            
            if(real_len_differ && !v1_path_len.containsKey(g1_v)) {
                sum += Math.abs(v2_path_len.get(g2_v));
            }
        }
        
        d_sim = 1.0 / (1 + string_sim + Math.abs(v1_neighbours - v2_neighbours) + sum);
        return d_sim;
    }
    
    
    /** Calculates similarity between two graphs, prepared and stored in map pairs.
     * @param vert_n1   number of vertices in first graph
     * @param vert_n2   number of vertices in second graph
     */
    public static double calcSimilarity(Map<Vertex,VertexPairSimilar> pairs,int vert_n1,int vert_n2) {
        
        double r = 0.0;
        for(VertexPairSimilar p:pairs.values()) {
            r += p.getSimilarity();
        }
   
        r = 2.0 * r / ((double)(vert_n1 +  vert_n2));
        
        return r;
    }
    
    

    /// //////////////
    /// Util functions 
    /// //////////////
    
    static final double DOUBLE_MIN = 0.001;
    
    /** Compares two double values.
     * @param eps The positive value of maximum error.
     * @return true if the difference is less then the value of <code>eps</code>.
     */
    public static boolean equalsDoubles(double x,double y, double eps) {
        return Math.abs(x-y) < eps;
    }
    public static boolean equalsDoubles(double x,double y) {
        return equalsDoubles(x,y,DOUBLE_MIN);
    }

    /** Gets vertex of graph by its name */
    public static Vertex getVertexByName(Graph g,String name) {
        for(Object o:g.getVertices()) {
            Vertex v = (Vertex)o;
            String s = (String)v.getUserDatum(GraphComparator.NAME_KEY);
            if(name.equalsIgnoreCase(s)) {
                return v;
            }
        }
        return null;
    }
    
    /** Gets vertex of graph by its name and subgraph number */
    public static Vertex getVertexByName(Graph g,String name,String subgraph_n) {
        for(Object o:g.getVertices()) {
            Vertex v = (Vertex)o;
            
            boolean b1 = null == subgraph_n;
            boolean b2 = v.containsUserDatumKey(GraphComparator.SUBGRAPH_N);
            boolean b3 = b2 &&
                        ((String)v.getUserDatum(GraphComparator.SUBGRAPH_N)).equalsIgnoreCase(subgraph_n);
            
            if(name.equalsIgnoreCase((String)v.getUserDatum(GraphComparator.NAME_KEY))) {
                if((!b1 && b3) || (b1 && !b2)) {
                    return v;
                }
            }
        }
        return null;
    }
    //boolean b1 = v1.containsUserDatumKey(GraphComparator.SUBGRAPH_N);
}


